A [Mapper](mapper.html) is an object that is used to construct a middleware that
calls the first app whose virtual location matches the initial portion of the
request URL. A location is simply a string that begins with a slash and
specifies the beginning portion of a URL. In order for a location to "match", it
must match the beginning of the request URL and the remainder of the URL (when
the location is removed) must be empty or begin with a slash.

You use a `Mapper` whenever you want to handle all requests to a certain
location with a given app. A `Mapper` tries all location/app pairs in the order
they are defined, and calls the first app whose location matches.

When a `Mapper` calls an app, it modifies the `scriptName` and `pathInfo`
environment variables such that the app's location is appended to the current
value of `scriptName` and stripped from the beginning of `pathInfo`. This allows
all downstream apps to function independent of location, while still allowing
them to reconstruct the full request URL as needed (e.g. when creating links).

The following example runs a sample app that demonstrates this behavior. It
"mounts" a simple app that returns the current value of the `scriptName` and
`pathInfo` environment variables in the HTTP headers in the response.

    var app = new strata.Mapper;

    // A simple app that returns the scriptName and pathInfo of the current
    // environment as X-ScriptName and X-PathInfo headers, respectively.
    function scriptNamePathInfo(env, callback) {
        callback(200, {
            "X-ScriptName": env.scriptName,
            "X-PathInfo": env.pathInfo
        }, "");
    }

    app.map("/images", scriptNamePathInfo);
    app.map("/files", scriptNamePathInfo);

    strata.run(app);

The `Mapper` API is described below.

    - new Mapper(app)

      Creates a new `Mapper` using the given `app` as the default app that is
      called when no other location matches (this is equivalent to an app having
      location "/" which essentially targets the server "root").

    - Mapper.fromMap(map, app)

      Creates a new `Mapper` using the given `app` as the default and all
      location/app pairs in the given `map`.

    - mapper.map(location, app)

      Maps the given `location` to the given `app`. When the beginning of the
      request URL's path matches the `location` (assuming no other previously
      registered app matches), the `app` will be called.

    - mapper.call(env, callback)

      Calls this `Mapper` as a Strata application.

    - mapper.toApp()

      Returns a Strata application that may be used to run this `Mapper`.
