A [Router](router.html) is an object that is used to construct a middleware that
calls the first app whose route matches the request. A route is simply a regular
expression and optionally a set of request methods. A route "matches" the
request when its regular expression matches the request URL and its request
method matches the method that was used in the request. A route that has no
request methods associated with it may match any method.

All routes that have a request method are tried first in the order they are
defined according to the method used in the request. Any routes that have no
request method are tried afterwards. Once a router finds an app whose route
matches, it calls that app and stops trying other routes. It also sets the
`strata.route` environment variable, which is an array that contains the result of
the match, including any captures.

The following app demonstrates how to retrieve the portion of the URL that was
matched by the route, as well as how to restrict routes to a certain request
method.

    var app = new strata.Router;

    // POST /users
    app.route(/\/users/, function (env, callback) {
        callback(200, {}, "Create a new user");
    }, "POST");

    // GET /users/(\d+)
    app.route(/\/users\/(\d+)/, function (env, callback) {
        var route = env["strata.route"];
        callback(200, {}, "Get user with id " + route[1]);
    }, "GET");

    // DELETE /users/(\d+)
    app.route(/\/users\/(\d+)/, function (env, callback) {
        var route = env["strata.route"];
        callback(200, {}, "Delete user with id " + route[1]);
    }, "DELETE");

In addition to regular expressions, a route may be defined using a string,
similar to the way that routes are defined in Rails or Sinatra. When using a
string, it is compiled to a regular expression using `Router.compileRoute`. This
allows you to use named route segments using the colon notation.

The following app is a duplicate of the above app using strings instead of
regular expressions to define the routes. Notice how route variables are able to
be retrieved using the name defined inside the route instead of their capture
index.

    var app = new strata.Router;

    // POST /users
    app.route("/users", function (env, callback) {
        callback(200, {}, "Create a new user");
    }, "POST");

    // GET /users/:id
    app.route("/users/:id", function (env, callback) {
        var route = env["strata.route"];
        callback(200, {}, "Get user with id " + route.id);
    }, "GET");

    // DELETE /users/:id
    app.route("/users/:id", function (env, callback) {
        var route = env["strata.route"];
        callback(200, {}, "Delete user with id " + route.id);
    }, "DELETE");

The `Router` API is described below.

    - new Router(app)
    - Router.compileRoute(route, keys)
    - router.route(pattern, app, methods)
    - router.call(env, callback)
    - router.toApp()
